@@ -1,10 +1,12 @@
 #	READ IMAGE ANALYSIS FILES INTO RGList
 
-read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,columns=NULL,other.columns=NULL,annotation=NULL,wt.fun=NULL,verbose=TRUE,sep="\t",quote="\"", DEBUG = FALSE, ...)
+read.maimages <- function(files=NULL,source="generic",path=NULL,ext=NULL,names=NULL,columns=NULL,other.columns=NULL,annotation=NULL,wt.fun=NULL,verbose=TRUE,sep="\t",quote=NULL,DEBUG=FALSE,...)
 #	Extracts an RG list from a series of image analysis output files
-#	Gordon Smyth. Use of colClasses added by Marcus Davy.
+#	Gordon Smyth. 
 #	1 Nov 2002.  Last revised 14 October 2005.
+#	Use of colClasses added by Marcus Davy, 14 October 2005.
 {
+#	For checking colClasses setup
 	debugVars <- function(DEBUG){
 		if(DEBUG) {
             print(paste("FILE:", fullname))
@@ -14,8 +16,10 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
         }
     }
 
+#	Begin check input arguments
+
 	if(missing(files)) {
-		if(missing(ext))
+		if(is.null(ext))
 			stop("Must specify input files")
 		else {
 			extregex <- paste("\\.",ext,"$",sep="")
@@ -23,19 +27,20 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
 			files <- sub(extregex,"",files)
 		}
 	}
-	if(is.null(source)) {
-		source <- "generic"
-	} else {
-		source <- match.arg(source,c("agilent","arrayvision","bluefuse","genepix","genepix.median","genepix.custom","imagene","quantarray","smd.old","smd","spot","spot.close.open"))
-		if(source=="imagene") return(read.imagene(files=files,path=path,ext=ext,names=names,columns=columns,wt.fun=wt.fun,verbose=verbose,sep=sep,quote=quote,...))
-	}
+
+	source <- match.arg(source,c("generic","agilent","arrayvision","bluefuse","genepix","genepix.median","genepix.custom","imagene","quantarray","smd.old","smd","spot","spot.close.open"))
+	source2 <- strsplit(source,split=".",fixed=TRUE)[[1]][1]
+	if(is.null(quote)) if(source=="agilent") quote <- "" else quote <- "\""
+	if(source2=="imagene") return(read.imagene(files=files,path=path,ext=ext,names=names,columns=columns,wt.fun=wt.fun,verbose=verbose,sep=sep,quote=quote,...))
+
 	slides <- as.vector(as.character(files))
 	if(!is.null(ext)) slides <- paste(slides,ext,sep=".")
 	nslides <- length(slides)
+
 	if(is.null(names)) names <- removeExt(files)
 
 	if(is.null(columns)) {
-		if(is.null(source)) stop("At least one of 'source' or 'columns' must be given")
+		if(source2=="generic") stop("must specify columns for generic input")
 		columns <- switch(source,
 			agilent = list(G="gMeanSignal",Gb="gBGMedianSignal",R="rMeanSignal",Rb="rBGMedianSignal"),
 			bluefuse = list(G="AMPCH1",R="AMPCH2"),
@@ -56,8 +61,8 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
 		if(is.null(columns$G) || is.null(columns$R)) stop("columns must specify foreground G and R")
 		if(!all(names(columns) %in% c("G","R","Gb","Rb"))) warning("non-standard columns specified")
 	}
+	cnames <- names(columns)
 	
-	#	Set annotation information
 	if(is.null(annotation)) annotation <- switch(source,
 		agilent = c("Row","Col","Start","Sequence","SwissProt","GenBank","Primate","GenPept","ProbeUID","ControlType","ProbeName","GeneName","SystematicName","Description"),
 		bluefuse = c("ROW","COL","SUBGRIDROW","SUBGRIDCOL","BLOCK","NAME","ID"),   
@@ -66,14 +71,14 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
 		smd.old = c("SPOT","NAME","Clone ID","Gene Symbol","Gene Name","Cluster ID","Accession","Preferred name","SUID"),
 		NULL
 	)
-	
-	cnames <- names(columns)
+
+#	End check input arguments
 
 #	Read first file to get nspots
 	fullname <- slides[1]
 	if(!is.null(path)) fullname <- file.path(path,fullname)
-	switch(source,
-	   quantarray = {
+	switch(source2,
+		quantarray = {
 		firstfield <- scan(fullname,what="",sep="\t",flush=TRUE,quiet=TRUE,blank.lines.skip=FALSE,multi.line=FALSE,allowEscapes=FALSE)
 		skip <- grep("Begin Data",firstfield)
 		if(length(skip)==0) stop("Cannot find \"Begin Data\" in image output file")
@@ -100,18 +105,18 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
 		allcnames <- scan(fullname, what=character(1), sep=sep, quote=quote, skip=skip, nlines=1, quiet=TRUE, allowEscapes=FALSE)
         colClasses <- getColClasses(allcnames, annotation, columns, other.columns, wt.fun)
         debugVars(DEBUG)
-		obj <- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,as.is=TRUE,check.names=FALSE,comment.char= "",fill=TRUE, colClasses=colClasses, ...)
+		obj <- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,as.is=TRUE,check.names=FALSE,comment.char="",fill=TRUE, colClasses=colClasses, ...)
 		nspots <- nrow(obj)
-	}, genepix=,genepix.median=,genepix.custom = {
+	}, genepix = {
 		h <- readGPRHeader(fullname)
 		if(verbose && source=="genepix.custom") cat("Custom background:",h$Background,"\n")
 		skip <- h$NHeaderRecords
 		allcnames <- scan(fullname, what=character(1), sep=sep, quote=quote, skip=skip, nlines=1, quiet=TRUE, allowEscapes=FALSE)
-        colClasses <- getColClasses(allcnames, annotation, columns, other.columns, wt.fun)
+        colClasses <- getColClasses(allcnames, annotation, columns, other.columns, wt.fun, "X")
         debugVars(DEBUG)
 		obj <- read.table(fullname,skip=skip,header=TRUE,sep=sep,quote=quote,as.is=TRUE,check.names=FALSE,comment.char="",fill=TRUE, colClasses=colClasses, ...)
 		nspots <- nrow(obj)
-	}, smd=,smd.old = {
+	}, smd = {
 		skip <- readSMDHeader(fullname)$NHeaderRecords
 		allcnames <- scan(fullname, what=character(1), sep=sep, quote=quote, skip=skip, nlines=1, quiet=TRUE, allowEscapes=FALSE)
         colClasses <- getColClasses(allcnames, annotation, columns, other.columns, wt.fun)
@@ -139,20 +144,41 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
 	if(!is.null(wt.fun)) RG$weights <- Y
 	RG$targets <- data.frame(FileName=I(files),row.names=names)
 
-
+#	Set annotation columns
 	if(!is.null(annotation)) {
 		j <- match(annotation,colnames(obj),0)
-		if(any(j>0)) RG$genes <- data.frame(obj[,j,drop=FALSE])
+		if(any(j>0)) RG$genes <- data.frame(obj[,j,drop=FALSE],check.names=FALSE)
 	}
 
-#	Set printer layout
-	if(source=="agilent") {
+	RG$source <- source
+
+#	Set printer layout, if possible
+	if(source2=="agilent") {
 		if(!is.null(RG$genes$Row) && !is.null(RG$genes$Col)) {
 			nr <- length(unique(RG$genes$Row))
 			nc <- length(unique(RG$genes$Col))
 			if(nspots==nr*nc) RG$printer <- list(ngrid.r=1,ngrid.c=1,nspot.r=nr,nspot.c=nc)
 		}
 	}
+	if(source2=="genepix") {
+		if(!is.null(RG$genes$Block) && !is.null(RG$genes$Row) && !is.null(RG$genes$Column)) {
+			RG$printer <- getLayout(RG$genes,guessdups=TRUE)
+			if(!is.null(obj$X)) {
+				blocksize <- RG$printer$nspot.r*RG$printer$nspot.c
+				nblocks <- RG$printer$ngrid.r*RG$printer$ngrid.c
+				i <- (1:(nblocks-1))*blocksize
+				ngrid.r <- sum(obj$X[i] > obj$X[i+1]) + 1
+				if(nblocks%%ngrid.r==0) {
+					RG$printer$ngrid.r <- ngrid.r
+					RG$printer$ngrid.c <- nblocks/ngrid.r
+				} else {
+					warning("Can't determine number of grid rows")
+					RG$printer$ngrid.r <- RG$ngrid.c <- NA
+				}
+			}
+		}
+		
+	}
 
 #	Other columns
 	if(!is.null(other.columns)) {
@@ -167,12 +193,12 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
 		}
 	}
 
-#	Now read remainder of files
+#	Read remainder of files
 	for (i in 1:nslides) {
 		if(i > 1) {
 			fullname <- slides[i]
 			if(!is.null(path)) fullname <- file.path(path,fullname)
-			switch(source,
+			switch(source2,
 			   quantarray = {
                    firstfield <- scan(fullname, what = "", sep = "\t", flush = TRUE, 
                                       quiet = TRUE, blank.lines.skip = FALSE, multi.line = FALSE, 
@@ -182,9 +208,9 @@ read.maimages <- function(files,source=NULL,path=NULL,ext=NULL,names=NULL,column
                    skip <- 1
                }, bluefuse = {
                    skip <- readBlueFuseHeader(fullname)$NHeaderRecords
-               }, genepix = , genepix.median = , genepix.custom = {
+               }, genepix = {
 				   skip <- readGPRHeader(fullname)$NHeaderRecords
-               }, smd = , smd.old = {
+               }, smd = {
                    skip <- readSMDHeader(fullname)$NHeaderRecords
                }, {
                    skip <- grep(protectMetachar(columns$R), readLines(fullname, n = 80)) -1
@@ -249,4 +275,4 @@ getColClasses <- function(cols, ...)
     x[ind] <- NA
   }
   x
-}
\ No newline at end of file
+}
